#!/bin/sh
# Perform a test compilation of the LysKOM server under Xenofarm.
# Copyright (C) 2002  Lysator Academic Computer Association.
#
# This file is part of the LysKOM server.
# 
# LysKOM is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 1, or (at your option) 
# any later version.
# 
# LysKOM is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License
# along with LysKOM; see the file COPYING.  If not, write to
# Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
# or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
# MA 02139, USA.
#
# Please mail bug reports to bug-lyskom@lysator.liu.se.

# To understand this file, you should look at the xenofarm file 
# projects/lyskom-server/source-transform.sh (see <URL:http://cvs.
# lysator.liu.se/viewcvs/viewcvs.cgi/xenofarm/projects/lyskom-server/
# source-transform.sh?cvsroot=xenofarm>).  It will attempt to unpack
# the distribution, configure it, compile, run tests, and collect
# information in the "r" directory.
#
# How can this file unpack the distribution when it is part of the
# distribution?  Read source-transform.sh and find out.
#
# Why is this file present here, and not in the Xenofarm CVS module?
# Because whenever anything here changes, we probably want to re-run
# the tests on the Xenofarm, and committing anything to the
# lyskom-server CVS module will trigger the creation of a source
# distribution.

cfgargs=
makeargs=
while [ $# -gt 0 ]
do
  case "$1" in
      --cfg)    shift; cfgargs="$1"; shift;;
      --make)   shift; makeargs="$1"; shift;;
      *)        echo $0: unsupported argument $1 >&2; exit 1;;
  esac
done

rm -rf r
mkdir r
exec > r/shlog.txt 2>&1

VERS=`echo lyskom-server*tar.gz|sed s/lyskom-server-//|sed s/.tar.gz//`
BASE=lyskom-server-$VERS

timeecho () {
    echo `TZ=UTC date|awk '{print $6 "-" $2 "-" $3 " " $4}'\
	|sed -e s/Jan/01/ -e s/Feb/02/ -e s/Mar/03/ -e s/Apr/04/ \
	     -e s/May/05/ -e s/Jun/06/ -e s/Jul/07/ -e s/Aug/08/ \
	     -e s/Sep/09/ -e s/Oct/10/ -e s/Nov/11/ -e s/Dec/12/ `: "$@"
}

log () {
    echo "$@" >> r/mainlog.txt
    date >> r/mainlog.txt
}

dotask() {
    important="$1"
    task="$2"
    cmd="$3"
    if test $status = good
    then
	log Begin $task
        timeecho Begin $task
        if sh -c "$cmd" > r/${task}log.txt 2>&1
        then
	    touch r/$task.pass
        else
	    timeecho FAIL: $task
	    touch r/$task.fail
	    if [ $important = 1 ]
	    then
	        status=${task}-failed
	    fi
        fi
    else
	echo status $status makes it impossible to perform this step \
	    > r/${task}log.txt
    fi
}

pfx=`pwd`/pfx

status=good

dotask 1 "unzip" "gzip -d $BASE.tar.gz"
dotask 1 "unpack" "tar xf $BASE.tar"
dotask 1 "cfg" "cd $BASE && ./configure -C --prefix=$pfx $cfgargs"
dotask 0 "fds" "cd $BASE/src/server/testsuite && make check-testfd"
dotask 0 "id-cc" "cd $BASE && make ident-cc"
dotask 0 "id-tx" "cd $BASE/doc && make ident-makeinfo"
dotask 1 "make" "cd $BASE && make $makeargs"

#
# "make check" requirements
#

checkdocok=true
pdfok=true
dviok=true
checkprgok=true

# We need "python".
if python -c ""
then
    :
else
    echo python not found >> r/ckdoclog.txt
    echo python not found >> r/ckprglog.txt
    checkdocok=false
    checkprgok=false
fi

# We need the socket module from "python".
if $checkprgok
then
    if python -c "import socket"
    then
        :
    else
        echo python cannot import socket: >> r/ckprglog.txt
        python -c "import socket" >> r/ckprglog.txt 2>&1
        checkprgok=false
    fi
fi

# We need "runtest".
if runtest --version
then
    :
else 
    echo runtest not found >> r/ckprglog.txt
    checkprgok=false
fi

# We ned "pdftex"
if pdftex --version
then
    :
else 
    echo pdftex not found >> r/pdflog.txt
    pdfok=false
fi

# We ned "texi2dvi"
if texi2dvi --version
then
    :
else 
    echo texi2dvi not found >> r/dvilog.txt
    dviok=false
fi

# We ned "tex"
if tex --version
then
    :
else 
    echo tex not found >> r/dvilog.txt
    dviok=false
fi

# No use to run pdf and dvi if check-doc fails.
oldstatus=$status

if $checkdocok
then
    dotask 1 "ckdoc" "cd $BASE/doc && make check-doc"
fi

if $dviok
then
    dotask 0 "dvi" "cd $BASE/doc && make dvi"
fi

if $pdfok
then
    dotask 0 "pdf" "cd $BASE/doc && make pdf"
fi

# Restore status: check is useful even if check-doc failed.
status=$oldstatus

if $checkprgok
then
    dotask 0 "ckprg" "cd $BASE/src && make check"
    if test "`uname -n`" = taylor && test -f r/ckprg.fail
    then
	mv r/ckprg.fail r/ckprg.warn
    fi
fi

dotask 1 "install" "cd $BASE && make install"

if [ -f r/install.pass ]
then
    log Xenofarm OK
    find pfx -type f -print | sort > r/installedfiles.txt
fi

# FIXME: run distcheck.
# FIXME: compare the contents of the distcheck-generated tar file
# with the one we distributed.

log Begin response assembly
timeecho Collecting results

# Check for warnings

if test -f r/cfg.pass
then
    egrep -i 'warning|\(w\)' r/cfglog.txt \
    > r/cfgwarn.txt
    if test `wc -l < r/cfgwarn.txt` -gt 0
    then
	mv r/cfg.pass r/cfg.warn
    fi
fi

if test -f r/make.pass
then
    egrep -i 'warning|\(w\)' r/makelog.txt \
|grep -v '".*1506-112 .W. Duplicate type qualifier "volatile" ignored.' \
|grep -v '/usr/include/bits/socket.h:.*: warning: cast increases required alignment of target type' \
|grep -v '/usr/include/sys/vnode.h:.*: warning: declaration of .free. shadows global declaration' \
|grep -v 'aux-item-def-scan\.c:.*: warning: .* defined but not used' \
|grep -v 'connections.c:.*: warning: duplicate .volatile.' \
|grep -v '"format" is an unrecognized __declspec attribute' \
|grep -v 'getopt.[hc]:.*: warning: function declaration isn.t a prototype' \
|grep -v 'getopt.c:451: warning: .indfound. might be used uninitialized in this function' \
|grep -v 'getopt.c:461: warning: comparison between signed and unsigned' \
|grep -v 'getopt1.c:.*: warning: function declaration isn.t a prototype' \
|grep -v 'ramkomd.c:188: warning: function declaration isn.t a prototype' \
|grep -v 'ramkomd.c:305: warning: function declaration isn.t a prototype' \
|grep -v 'sigflags.h:3[06]: warning: duplicate .volatile.' \
|grep -v regex\\.c \
    > r/makewarn.txt
    if test `wc -l < r/makewarn.txt` -gt 0
    then
	mv r/make.pass r/make.warn
    else
	rm r/makewarn.txt
    fi
fi

if test -f r/ckprg.pass
then
    egrep -i 'warning|\(w\)|error' r/ckprglog.txt \
|grep -v 'WARNING: Couldn.t find the global config file.' \
|grep -v 'WARNING: Couldn.t find tool init file' \
    > r/ckprgwarn.txt
    if test `wc -l < r/ckprgwarn.txt` -gt 0
    then
	mv r/ckprg.pass r/ckprg.warn
	egrep -i 'error' r/ckprgwarn.txt \
	> r/ckprgfail.txt
	if test `wc -l < r/ckprgfail.txt` -gt 0
	then
	    mv r/ckprg.warn r/ckprg.fail
	else
	    rm r/ckprgfail.txt
	fi
    else
	rm r/ckprgwarn.txt
    fi
fi

# Collect stuff.

cp $BASE/config.cache r/configcache.txt
cp $BASE/config.log r/configlog.txt
cp $BASE/src/libraries/libisc-new/config.log r/iscconfiglog.txt
cp $BASE/config.h r/config-h.txt
for file in $BASE/src/server/testsuite/*.log
do
  if test -f $file
  then
      cp $file r/`basename $file`.txt
  fi
done
# find $BASE -name core -print
env > r/environ.txt
echo $PATH > r/path.txt
makeinfo --version > r/makeinfo.txt
type makeinfo >> r/makeinfo.txt 2>&1

cp buildid.txt r/buildid.txt

(cd r && tar cf - *) > xenofarm_result.tar
gzip -1 xenofarm_result.tar

exit 0
